@@ -42,6 +42,7 @@ from esmvaltool.diag_scripts.shared import (
     plot,
     run_diagnostic,
     select_metadata,
+    sorted_metadata,
 )
 
 logger = logging.getLogger(os.path.basename(__file__))
@@ -80,7 +81,7 @@ def _get_ancestor_files(cfg, obs_name, projects=None):
             select_metadata(cfg['input_data'].values(), project=project))
     datasets.extend(
         select_metadata(cfg['input_data'].values(), dataset=obs_name))
-    return [d['filename'] for d in datasets]
+    return sorted([d['filename'] for d in datasets])
 
 
 def _get_model_color(model, lambda_cube):
@@ -133,7 +134,7 @@ def _get_project(cfg):
     projects = [p for p in projects if 'obs' not in p.lower()]
     if len(projects) == 1:
         return projects[0]
-    return projects
+    return sorted(projects)
 
 
 def _save_fig(cfg, basename, legend=None):
@@ -156,6 +157,7 @@ def get_external_cubes(cfg):
     """Get external cubes for psi, ECS and lambda."""
     cubes = iris.cube.CubeList()
     input_data = list(cfg['input_data'].values())
+    input_data = sorted_metadata(input_data, ['short_name', 'exp', 'dataset'])
     for filename in ('psi.nc', 'ecs.nc', 'lambda.nc'):
         filepath = io.get_ancestor_file(cfg, filename)
         cube = iris.load_cube(filepath)
@@ -254,11 +256,12 @@ def plot_temperature_anomaly(cfg, tas_cubes, lambda_cube, obs_name):
     legend = _get_line_plot_legend()
 
     # Save plot
-    provenance_record['plot_file'] = _save_fig(cfg, filename, legend)
+    plot_path = _save_fig(cfg, filename, legend)
 
     # Write provenance
     with ProvenanceLogger(cfg) as provenance_logger:
         provenance_logger.log(netcdf_path, provenance_record)
+        provenance_logger.log(plot_path, provenance_record)
 
 
 def plot_psi(cfg, psi_cubes, lambda_cube, obs_name):
@@ -301,11 +304,12 @@ def plot_psi(cfg, psi_cubes, lambda_cube, obs_name):
     legend = _get_line_plot_legend()
 
     # Save plot
-    provenance_record['plot_file'] = _save_fig(cfg, filename, legend)
+    plot_path = _save_fig(cfg, filename, legend)
 
     # Write provenance
     with ProvenanceLogger(cfg) as provenance_logger:
         provenance_logger.log(netcdf_path, provenance_record)
+        provenance_logger.log(plot_path, provenance_record)
 
 
 def plot_emergent_relationship(cfg, psi_cube, ecs_cube, lambda_cube, obs_cube):
@@ -371,11 +375,12 @@ def plot_emergent_relationship(cfg, psi_cube, ecs_cube, lambda_cube, obs_cube):
     legend = AXES.legend(loc='upper left')
 
     # Save plot
-    provenance_record['plot_file'] = _save_fig(cfg, filename, legend)
+    plot_path = _save_fig(cfg, filename, legend)
 
     # Write provenance
     with ProvenanceLogger(cfg) as provenance_logger:
         provenance_logger.log(netcdf_path, provenance_record)
+        provenance_logger.log(plot_path, provenance_record)
 
 
 def plot_pdf(cfg, ecs_lin, ecs_pdf, ecs_cube, obs_name):
@@ -416,11 +421,12 @@ def plot_pdf(cfg, ecs_lin, ecs_pdf, ecs_cube, obs_name):
     legend = AXES.legend(loc='upper left')
 
     # Save plot
-    provenance_record['plot_file'] = _save_fig(cfg, filename, legend)
+    plot_path = _save_fig(cfg, filename, legend)
 
     # Write provenance
     with ProvenanceLogger(cfg) as provenance_logger:
         provenance_logger.log(netcdf_path, provenance_record)
+        provenance_logger.log(plot_path, provenance_record)
 
 
 def plot_cdf(cfg, ecs_lin, ecs_pdf, ecs_cube, obs_name):
@@ -473,11 +479,12 @@ def plot_cdf(cfg, ecs_lin, ecs_pdf, ecs_cube, obs_name):
     legend = AXES.legend(loc='upper left')
 
     # Save plot
-    provenance_record['plot_file'] = _save_fig(cfg, filename, legend)
+    plot_path = _save_fig(cfg, filename, legend)
 
     # Write provenance
     with ProvenanceLogger(cfg) as provenance_logger:
         provenance_logger.log(netcdf_path, provenance_record)
+        provenance_logger.log(plot_path, provenance_record)
 
 
 def get_ecs_range(cfg, ecs_lin, ecs_pdf):
@@ -504,6 +511,7 @@ def main(cfg):
     input_data = (
         select_metadata(cfg['input_data'].values(), short_name='tas') +
         select_metadata(cfg['input_data'].values(), short_name='tasa'))
+    input_data = sorted_metadata(input_data, ['short_name', 'exp', 'dataset'])
     if not input_data:
         raise ValueError("This diagnostics needs 'tas' or 'tasa' variable")
 
