@@ -38,6 +38,7 @@ import numpy as np
 
 from esmvaltool.diag_scripts.ocean import diagnostic_tools as diagtools
 from esmvaltool.diag_scripts.shared import run_diagnostic
+from esmvaltool.diag_scripts.shared._base import ProvenanceLogger
 
 # This part sends debug statements to stdout
 logger = logging.getLogger(os.path.basename(__file__))
@@ -173,7 +174,7 @@ def make_cube_region_dict(cube):
             layers.append(coord)
 
     cubes = {}
-    if layers == []:
+    if not layers:
         cubes[''] = cube
         return cubes
 
@@ -260,7 +261,7 @@ def make_transects_plots(
         qplt.contourf(cube, 15, linewidth=0, rasterized=True)
 
         if set_y_logscale:
-            plt.axes().set_yscale('log')
+            plt.gca().set_yscale('log')
 
         if region:
             region_title = region
@@ -288,12 +289,22 @@ def make_transects_plots(
             )
 
         # Saving files:
-        if cfg['write_plots']:
-            logger.info('Saving plots to %s', path)
-            plt.savefig(path)
-
+        logger.info('Saving plots to %s', path)
+        plt.savefig(path)
         plt.close()
 
+        provenance_record = diagtools.prepare_provenance_record(
+            cfg,
+            caption=f'Transect of {title}',
+            statistics=['mean'],
+            domain=['reg'],
+            plot_type=['sect', 'zonal', ],
+            ancestors=[filename],
+        )
+
+        with ProvenanceLogger(cfg) as provenance_logger:
+            provenance_logger.log(path, provenance_record)
+
 
 def add_sea_floor(cube):
     """
@@ -370,9 +381,9 @@ def make_transect_contours(
             linestyles=linestyles,
             rasterized=True)
 
-        # Determine y log scale.
+        # Determine y log scale. Use gca to set scale
         if determine_set_y_logscale(cfg, metadata):
-            plt.axes().set_yscale('log')
+            plt.gca().set_yscale('log')
 
         add_sea_floor(cube)
 
@@ -403,12 +414,22 @@ def make_transect_contours(
             )
 
         # Saving files:
-        if cfg['write_plots']:
-            logger.info('Saving plots to %s', path)
-            plt.savefig(path)
-
+        logger.info('Saving plots to %s', path)
+        plt.savefig(path)
         plt.close()
 
+        provenance_record = diagtools.prepare_provenance_record(
+            cfg,
+            caption=f'Transect of {title}',
+            statistics=['mean'],
+            domain=['reg'],
+            plot_type=['sect', 'zonal', ],
+            ancestors=[filename],
+        )
+
+        with ProvenanceLogger(cfg) as provenance_logger:
+            provenance_logger.log(path, provenance_record)
+
 
 def multi_model_contours(
         cfg,
@@ -496,7 +517,7 @@ def multi_model_contours(
             }
 
             if set_y_logscale:
-                plt.axes().set_yscale('log')
+                plt.gca().set_yscale('log')
 
             title = metadatas[filename]['long_name']
             units = str(model_cubes[filename][region].units)
@@ -513,20 +534,19 @@ def multi_model_contours(
         plt.legend(loc='best')
 
         # Saving files:
-        if cfg['write_plots']:
-            path = diagtools.get_image_path(
-                cfg,
-                metadatas[filename],
-                prefix='MultipleModels',
-                suffix='_'.join([
-                    'contour_tramsect', region,
-                    str(threshold) + image_extention
-                ]),
-                metadata_id_list=[
-                    'field', 'short_name', 'preprocessor', 'diagnostic',
-                    'start_year', 'end_year'
-                ],
-            )
+        path = diagtools.get_image_path(
+            cfg,
+            metadatas[filename],
+            prefix='MultipleModels',
+            suffix='_'.join([
+                'contour_tramsect', region,
+                str(threshold) + image_extention
+            ]),
+            metadata_id_list=[
+                'field', 'short_name', 'preprocessor', 'diagnostic',
+                'start_year', 'end_year'
+            ],
+        )
 
         # Resize and add legend outside thew axes.
         plt.gcf().set_size_inches(9., 6.)
@@ -537,6 +557,18 @@ def multi_model_contours(
         plt.savefig(path)
         plt.close()
 
+        provenance_record = diagtools.prepare_provenance_record(
+            cfg,
+            caption=f'Transect of {title}',
+            statistics=['mean'],
+            domain=['reg'],
+            plot_type=['sect', 'zonal', ],
+            ancestors=list(metadatas.keys()),
+        )
+
+        with ProvenanceLogger(cfg) as provenance_logger:
+            provenance_logger.log(path, provenance_record)
+
 
 def main(cfg):
     """
