@@ -1,24 +1,23 @@
 """Sea ice drift diagnostic."""
-import os
+import csv
 import logging
 import math
-import csv
+import os
 import warnings
 
-import numpy as np
-from scipy import stats
-from matplotlib import pyplot as plt
-
 import iris
-import iris.cube
 import iris.analysis
 import iris.analysis.cartography
 import iris.coords
-from iris.util import broadcast_to_shape
+import iris.cube
+import numpy as np
+import shapely
 from iris.aux_factory import AuxCoordFactory
+from iris.util import broadcast_to_shape
+from matplotlib import pyplot as plt
 from pyproj import Transformer
-from shapely.geometry import Polygon, Point
-
+from scipy import stats
+from shapely.geometry import Point, Polygon
 
 import esmvaltool.diag_scripts.shared
 import esmvaltool.diag_scripts.shared.names as n
@@ -33,7 +32,6 @@ warnings.filterwarnings("ignore", category=DeprecationWarning)
 
 class SeaIceDrift():
     """Class to compute SeaIce Drift metric."""
-
     def __init__(self, cfg):
         self.cfg = cfg
         self.datasets = esmvaltool.diag_scripts.shared.Datasets(self.cfg)
@@ -56,39 +54,33 @@ class SeaIceDrift():
         self.error_drift_sivol = {}
 
     def compute(self):
-        """Compute metric"""
+        """Compute metric."""
         logger.info('Loading sea ice concentration')
         siconc_original = {}
         siconc_files = self.datasets.get_path_list(
             standard_name='sea_ice_area_fraction')
         for filename in siconc_files:
             reference_dataset = self._get_reference_dataset(
-                'sic',
-                self.datasets.get_info('reference_dataset', filename)
-            )
+                'sic', self.datasets.get_info('reference_dataset', filename))
             alias = self._get_alias(filename, reference_dataset)
             siconc = iris.load_cube(filename, 'sea_ice_area_fraction')
             siconc.convert_units('1.0')
             siconc_original[alias] = siconc
 
             self.siconc[alias] = self._compute_mean(
-                siconc, self._get_mask(siconc, filename)
-            )
+                siconc, self._get_mask(siconc, filename))
 
         logger.info('Loading sea ice thickness')
         sithick_files = self.datasets.get_path_list(
             standard_name='sea_ice_thickness')
         for filename in sithick_files:
             reference_dataset = self._get_reference_dataset(
-                'sithick',
-                self.datasets.get_info('reference_dataset', filename)
-            )
+                'sithick', self.datasets.get_info('reference_dataset',
+                                                  filename))
             alias = self._get_alias(filename, reference_dataset)
             sithick = iris.load_cube(filename, 'sea_ice_thickness')
             self.sivol[alias] = self._compute_mean(
-                sithick,
-                self._get_mask(sithick, filename)
-            )
+                sithick, self._get_mask(sithick, filename))
 
         logger.info('Load sea ice velocities')
         sispeed_files = self.datasets.get_path_list(
@@ -96,34 +88,24 @@ class SeaIceDrift():
         obs_file = self.cfg.get('sispeed_obs', '')
         for filename in sispeed_files:
             reference_dataset = self._get_reference_dataset(
-                'sispeed',
-                self.datasets.get_info('reference_dataset', filename)
-            )
+                'sispeed', self.datasets.get_info('reference_dataset',
+                                                  filename))
             alias = self._get_alias(filename, reference_dataset)
             if obs_file and alias == 'reference':
                 obs_data = np.load(obs_file)
                 obs_data = obs_data.reshape((12, 35), order='F')
                 logger.debug(obs_data)
-                sispeed = iris.cube.Cube(
-                    obs_data,
-                    'sea_ice_speed',
-                    units='km day-1'
-                )
+                sispeed = iris.cube.Cube(obs_data,
+                                         'sea_ice_speed',
+                                         units='km day-1')
                 sispeed.add_dim_coord(
-                    iris.coords.DimCoord(
-                        range(1, 13), var_name='month_number'
-                    ),
-                    0
-                )
+                    iris.coords.DimCoord(range(1, 13),
+                                         var_name='month_number'), 0)
                 sispeed.add_dim_coord(
-                    iris.coords.DimCoord(
-                        range(1979, 1979 + 35), var_name='year'
-                    ),
-                    1
-                )
+                    iris.coords.DimCoord(range(1979, 1979 + 35),
+                                         var_name='year'), 1)
                 sispeed.extract(
-                    iris.Constraint(year=lambda c: 1979 <= c <= 2005)
-                )
+                    iris.Constraint(year=lambda c: 1979 <= c <= 2005))
                 sispeed = sispeed.collapsed('year', iris.analysis.MEAN)
                 logger.debug(sispeed)
                 self.sispeed[alias] = sispeed
@@ -131,8 +113,7 @@ class SeaIceDrift():
                 sispeed = iris.load_cube(filename, 'sea_ice_speed')
                 sispeed.convert_units('km day-1')
                 self.sispeed[alias] = self._compute_mean(
-                    sispeed, self._get_mask(sispeed, filename)
-                )
+                    sispeed, self._get_mask(sispeed, filename))
 
         self._compute_metrics()
         self._results()
@@ -171,8 +152,7 @@ class SeaIceDrift():
 
         dataset_info = self.datasets.get_dataset_info(filename)
         var_info = esmvaltool.diag_scripts.shared.group_metadata(
-            self.cfg['input_data'].values(), 'alias'
-        )[dataset_info[n.ALIAS]]
+            self.cfg['input_data'].values(), 'alias')[dataset_info[n.ALIAS]]
         var_info = esmvaltool.diag_scripts.shared.group_metadata(
             var_info, 'short_name')
         if 'areacello' in var_info:
@@ -200,8 +180,7 @@ class SeaIceDrift():
             logger.debug('Sispeed: %s', self.sispeed[dataset].data)
             logger.info('Slope ratio (no unit)')
             slope, intercept, _, _ = self._get_slope_ratio(
-                self.sivol[dataset], self.sispeed[dataset]
-            )
+                self.sivol[dataset], self.sispeed[dataset])
             self.slope_drift_sivol[dataset] = slope
             self.intercept_drift_sivol[dataset] = intercept
 
@@ -212,12 +191,10 @@ class SeaIceDrift():
             logger.info('Compute mean errors (%)')
             self.error_drift_siconc[dataset] = self._compute_error(
                 self.siconc[dataset], self.siconc['reference'],
-                self.sispeed[dataset], self.siconc['reference']
-            )
+                self.sispeed[dataset], self.siconc['reference'])
             self.error_drift_sivol[dataset] = self._compute_error(
                 self.sivol[dataset], self.sivol['reference'],
-                self.sispeed[dataset], self.siconc['reference']
-            )
+                self.sispeed[dataset], self.siconc['reference'])
 
             logger.info('Compute relative slope ratios ')
             self.slope_ratio_drift_siconc[dataset] = \
@@ -241,15 +218,13 @@ class SeaIceDrift():
     @staticmethod
     def _compute_mean(data, weights):
         mapping = set(
-            data.coord_dims('latitude') + data.coord_dims('longitude')
-        )
+            data.coord_dims('latitude') + data.coord_dims('longitude'))
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
             return data.collapsed(
                 ('latitude', 'longitude'),
                 iris.analysis.MEAN,
-                weights=broadcast_to_shape(weights, data.shape, mapping)
-            )
+                weights=broadcast_to_shape(weights, data.shape, mapping))
 
     @staticmethod
     def _compute_error(var, var_obs, drift, drift_obs):
@@ -258,23 +233,23 @@ class SeaIceDrift():
         drift = drift.data
         drift_obs = drift_obs.data
 
-        return 100. * np.nanmean(np.sqrt(
-            SeaIceDrift._var_error(var, var_obs) +
-            SeaIceDrift._var_error(drift, drift_obs)
-        ))
+        return 100. * np.nanmean(
+            np.sqrt(
+                SeaIceDrift._var_error(var, var_obs) +
+                SeaIceDrift._var_error(drift, drift_obs)))
 
     @staticmethod
     def _var_error(var, obs):
         var_error = np.absolute(var - obs)
         var_mean = np.nanmean(obs)
         var_error_normal = var_error / var_mean
-        return var_error_normal ** 2
+        return var_error_normal**2
 
     @staticmethod
     def _get_slope_ratio(siconc, drift):
         slope, intercept = np.polyfit(siconc.data, drift.data, 1)
-        std_dev, sig = SeaIceDrift._sd_slope(
-            slope, intercept, siconc.data, drift.data)
+        std_dev, sig = SeaIceDrift._sd_slope(slope, intercept, siconc.data,
+                                             drift.data)
         return slope, intercept, std_dev, sig
 
     @staticmethod
@@ -286,8 +261,8 @@ class SeaIceDrift():
 
         # Compute standard deviation of slope
         lreg = slope * sivar + intercept  # linear regression
-        s_yx = np.sum((drift - lreg) ** 2) / (MONTHS_PER_YEAR - 2)
-        ss_xx = np.sum((sivar - np.mean(sivar)) ** 2)
+        s_yx = np.sum((drift - lreg)**2) / (MONTHS_PER_YEAR - 2)
+        ss_xx = np.sum((sivar - np.mean(sivar))**2)
         sd_slope = np.sqrt(s_yx / ss_xx)  # Standard deviation of slope
 
         # Significance
@@ -308,22 +283,17 @@ class SeaIceDrift():
             return
         logger.info('Dataset %s', model)
         if 'latitude_treshold' in self.cfg:
-            logger.info(
-                'Metrics computed over domain north of %s',
-                self.cfg['latitude_treshold']
-            )
+            logger.info('Metrics computed over domain north of %s',
+                        self.cfg['latitude_treshold'])
         else:
-            logger.info(
-                'Metrics computed inside %s region',
-                self.cfg.get('polygon_name', 'SCICEX')
-            )
+            logger.info('Metrics computed inside %s region',
+                        self.cfg.get('polygon_name', 'SCICEX'))
         logger.info('Slope ratio Drift-Concentration = {0:.3}'
                     ''.format(self.slope_ratio_drift_siconc[model]))
         logger.info('Mean error Drift-Concentration (%) = {0:.4}'
                     ''.format(self.error_drift_siconc[model]))
         logger.info('Slope ratio Drift-Thickness = {0:.3}'.format(
-            self.slope_ratio_drift_sivol.get(model, math.nan)
-        ))
+            self.slope_ratio_drift_sivol.get(model, math.nan)))
         logger.info('Mean error Drift-Thickness (%) = {0:.4}'
                     ''.format(self.error_drift_sivol.get(model, math.nan)))
 
@@ -344,14 +314,12 @@ class SeaIceDrift():
 
         with open(siconc_path, 'w') as csvfile:
             csv_writer = csv.writer(csvfile)
-            csv_writer.writerow(('slope', 'intercept',
-                                 'slope_ratio', 'error'))
-            csv_writer.writerow((
-                self.slope_drift_sic[dataset],
-                self.intercept_drift_siconc[dataset],
-                self.slope_ratio_drift_siconc.get(dataset, None),
-                self.error_drift_siconc.get(dataset, None)
-            ))
+            csv_writer.writerow(('slope', 'intercept', 'slope_ratio', 'error'))
+            csv_writer.writerow(
+                (self.slope_drift_sic[dataset],
+                 self.intercept_drift_siconc[dataset],
+                 self.slope_ratio_drift_siconc.get(dataset, None),
+                 self.error_drift_siconc.get(dataset, None)))
 
         sic_data, ancestors_sic = self._get_data_and_ancestors(dataset, 'sic')
         _, ancestors_sispeed = self._get_data_and_ancestors(dataset, 'sispeed')
@@ -359,31 +327,25 @@ class SeaIceDrift():
             dataset, 'sithick')
         caption = (
             f"Drift - siconc metric between {sic_data[n.START_YEAR]} and "
-            f"{sic_data[n.END_YEAR]} according to {dataset}"
-        )
-        self._create_prov_record(
-            siconc_path, caption, ancestors_sic + ancestors_sispeed)
+            f"{sic_data[n.END_YEAR]} according to {dataset}")
+        self._create_prov_record(siconc_path, caption,
+                                 ancestors_sic + ancestors_sispeed)
 
         with open(sivol_path, 'w') as csvfile:
             csv_writer = csv.writer(csvfile)
-            csv_writer.writerow(('slope', 'intercept',
-                                 'slope_ratio', 'error'))
-            csv_writer.writerow((
-                self.slope_drift_sivol[dataset],
-                self.intercept_drift_sivol[dataset],
-                self.slope_ratio_drift_sivol.get(dataset, None),
-                self.error_drift_sivol.get(dataset, None)
-            ))
+            csv_writer.writerow(('slope', 'intercept', 'slope_ratio', 'error'))
+            csv_writer.writerow(
+                (self.slope_drift_sivol[dataset],
+                 self.intercept_drift_sivol[dataset],
+                 self.slope_ratio_drift_sivol.get(dataset, None),
+                 self.error_drift_sivol.get(dataset, None)))
         caption = (
             f"Drift - sithick metric between {sithick_data[n.START_YEAR]} and "
-            f"{sithick_data[n.END_YEAR]} according to {dataset}"
-        )
-        self._create_prov_record(
-            sivol_path, caption, ancestors_sithick + ancestors_sispeed)
+            f"{sithick_data[n.END_YEAR]} according to {dataset}")
+        self._create_prov_record(sivol_path, caption,
+                                 ancestors_sithick + ancestors_sispeed)
 
     def _plot_results(self):
-        if not self.cfg[n.WRITE_PLOTS]:
-            return
         logger.info('Plotting results')
         for model in self.siconc:
             if model == 'reference':
@@ -401,32 +363,26 @@ class SeaIceDrift():
             os.makedirs(base_path)
         plot_path = os.path.join(
             base_path,
-            'drift-strength.{0}'.format(self.cfg[n.OUTPUT_FILE_TYPE])
-        )
+            'drift-strength.{0}'.format(self.cfg[n.OUTPUT_FILE_TYPE]))
         fig.savefig(plot_path)
 
         sic_data, ancestors_sic = self._get_data_and_ancestors(dataset, 'sic')
         _, ancestors_sispeed = self._get_data_and_ancestors(dataset, 'sispeed')
         _, ancestors_sithick = self._get_data_and_ancestors(dataset, 'sithick')
 
-        _, ancestors_sic_ref = self._get_data_and_ancestors(
-            'reference', 'sic')
+        _, ancestors_sic_ref = self._get_data_and_ancestors('reference', 'sic')
         _, ancestors_sispeed_ref = self._get_data_and_ancestors(
             'reference', 'sispeed')
         _, ancestors_sithick_ref = self._get_data_and_ancestors(
             'reference', 'sithick')
 
-        caption = (
-            "Drift - sithick and drift - siconc plot between "
-            f"{sic_data[n.START_YEAR]} and "
-            f"{sic_data[n.END_YEAR]} for {dataset} and reference"
-        )
+        caption = ("Drift - sithick and drift - siconc plot between "
+                   f"{sic_data[n.START_YEAR]} and "
+                   f"{sic_data[n.END_YEAR]} for {dataset} and reference")
         self._create_prov_record(
-            plot_path,
-            caption,
+            plot_path, caption,
             ancestors_sic + ancestors_sispeed + ancestors_sithick +
-            ancestors_sic_ref + ancestors_sispeed_ref + ancestors_sithick_ref
-        )
+            ancestors_sic_ref + ancestors_sispeed_ref + ancestors_sithick_ref)
 
     def _plot_drift_sivol(self, axes, dataset):
         drift = self.sispeed[dataset].data
@@ -444,14 +400,18 @@ class SeaIceDrift():
         drift_obs = self.sispeed['reference'].data
         sivol_obs = self.sivol['reference'].data
 
-        axes.plot([sivol[-1], sivol[0]], [drift[-1], drift[0]], 'r-',
+        axes.plot([sivol[-1], sivol[0]], [drift[-1], drift[0]],
+                  'r-',
                   linewidth=2)
         axes.plot(sivol, drift, 'ro-', label='model', linewidth=2)
-        axes.plot(sivol, slope_sivol * sivol + intercept_sivol, 'r:',
+        axes.plot(sivol,
+                  slope_sivol * sivol + intercept_sivol,
+                  'r:',
                   linewidth=2)
 
         axes.plot([sivol_obs[-1], sivol_obs[0]], [drift_obs[-1], drift_obs[0]],
-                  'b-', linewidth=2)
+                  'b-',
+                  linewidth=2)
         axes.plot(
             sivol_obs,
             drift_obs,
@@ -461,10 +421,11 @@ class SeaIceDrift():
             # #   r'; $\epsilon_h$=' +
             # #   str(np.round(error_sivol, 1)) +
             #   r')',
-            linewidth=2
-        )
-        axes.plot(sivol_obs, slope_sivol_obs * sivol_obs + intercept_sivol_obs,
-                  'b:', linewidth=2)
+            linewidth=2)
+        axes.plot(sivol_obs,
+                  slope_sivol_obs * sivol_obs + intercept_sivol_obs,
+                  'b:',
+                  linewidth=2)
 
         axes.set_xlabel('Sea ice thickness (m)', fontsize=18)
         axes.set_ylabel('Sea ice drift speed (km d$^{-1}$)', fontsize=18)
@@ -493,18 +454,16 @@ class SeaIceDrift():
         siconc_obs = self.siconc['reference'].data
 
         axes.plot(siconc, drift, 'ro', label='model')
-        axes.plot(siconc, slope_siconc * siconc + intercept_siconc, 'r:',
+        axes.plot(siconc,
+                  slope_siconc * siconc + intercept_siconc,
+                  'r:',
                   linewidth=2)
 
-        axes.plot(
-            siconc_obs,
-            drift_obs,
-            'bo',
-            label='reference'
-        )
-        axes.plot(siconc_obs, slope_siconc_obs * siconc_obs +
-                  intercept_siconc_obs,
-                  'b:', linewidth=2)
+        axes.plot(siconc_obs, drift_obs, 'bo', label='reference')
+        axes.plot(siconc_obs,
+                  slope_siconc_obs * siconc_obs + intercept_siconc_obs,
+                  'b:',
+                  linewidth=2)
 
         axes.set_xlabel('Sea ice concentration', fontsize=18)
         axes.set_ylabel('Sea ice drift speed (km d$^{-1}$)', fontsize=18)
@@ -520,8 +479,11 @@ class SeaIceDrift():
     @staticmethod
     def _annotate_points(axes, xvalues, yvalues):
         for i, j, k in zip(xvalues, yvalues, range(1, 12 + 1)):
-            axes.annotate(k, xy=(i, j), xytext=(10, 5),
-                          ha='right', textcoords='offset points')
+            axes.annotate(k,
+                          xy=(i, j),
+                          xytext=(10, 5),
+                          ha='right',
+                          textcoords='offset points')
 
     @staticmethod
     def _get_plot_limits(sivol, sivol_obs, step=0.55):
@@ -548,20 +510,15 @@ class SeaIceDrift():
             dataset = self.references[var]
 
         data = esmvaltool.diag_scripts.shared.group_metadata(
-            self.cfg['input_data'].values(), 'alias'
-        )[dataset]
+            self.cfg['input_data'].values(), 'alias')[dataset]
         data = esmvaltool.diag_scripts.shared.group_metadata(
-            data, 'short_name'
-        )
-        return (
-            data[var][0],
-            [data[var][0]['filename'], data['areacello'][0]['filename']]
-        )
+            data, 'short_name')
+        return (data[var][0],
+                [data[var][0]['filename'], data['areacello'][0]['filename']])
 
 
 class InsidePolygonFactory(AuxCoordFactory):
     """Defines a coordinate."""
-
     def __init__(self, polygon=None, lat=None, lon=None):
         """
         Args:
@@ -582,10 +539,8 @@ class InsidePolygonFactory(AuxCoordFactory):
         self.attributes = {}
 
         polygon.append(polygon[0])
-        self.transformer = Transformer.from_crs(
-            "WGS84",
-            "North_Pole_Stereographic"
-        )
+        self.transformer = Transformer.from_crs("WGS84",
+                                                "North_Pole_Stereographic")
 
         transformed = []
         for lon_val, lat_val in polygon:
@@ -594,26 +549,28 @@ class InsidePolygonFactory(AuxCoordFactory):
 
     @property
     def dependencies(self):
-        """
-        Return a dict mapping from constructor names to coordinates.
-        """
+        """Return a dict mapping from constructor names to coordinates."""
         return {'lat': self.lat, 'lon': self.lon}
 
     def _derive(self, lat, lon):
         def in_polygon(lat, lon):
-            """Check if point is inside polygon"""
+            """Check if point is inside polygon."""
             if lon > 180:
                 lon -= 360
             point = self.transformer.transform(lon, lat)
-            if self.polygon.contains(Point(point[0], point[1])):
+            try:
+                contained = self.polygon.contains(Point(point[0], point[1]))
+            except shapely.errors.TopologicalError:
+                return np.nan
+            if contained:
                 return 1.
             return np.nan
+
         vectorized = np.vectorize(in_polygon)
         return vectorized(lat, lon)
 
     def make_coord(self, coord_dims_func):
-        """
-        Returns a new :class:`iris.coords.AuxCoord`
+        """Returns a new :class:`iris.coords.AuxCoord`
 
         Args:
         * coord_dims_func:
@@ -627,25 +584,23 @@ class InsidePolygonFactory(AuxCoordFactory):
 
         # Build the points array.
         nd_points_by_key = self._remap(dependency_dims, derived_dims)
-        points = self._derive(nd_points_by_key['lat'],
-                              nd_points_by_key['lon'],)
-
-        in_polygon = iris.coords.AuxCoord(
-            points,
-            standard_name=self.standard_name,
-            long_name=self.long_name,
-            var_name=self.var_name,
-            units=self.units,
-            bounds=None,
-            attributes=self.attributes,
-            coord_system=self.coord_system
+        points = self._derive(
+            nd_points_by_key['lat'],
+            nd_points_by_key['lon'],
         )
+
+        in_polygon = iris.coords.AuxCoord(points,
+                                          standard_name=self.standard_name,
+                                          long_name=self.long_name,
+                                          var_name=self.var_name,
+                                          units=self.units,
+                                          bounds=None,
+                                          attributes=self.attributes,
+                                          coord_system=self.coord_system)
         return in_polygon
 
     def update(self, old_coord, new_coord=None):
-        """
-        Notify factory about the removal/replacement of a coordinate
-
+        """Notify factory about the removal/replacement of a coordinate.
 
         Args:
         * old_coord:
