@@ -33,6 +33,7 @@ END_YEAR = 'end_year'
 
 INPUT_DATA = 'input_data'
 INPUT_FILES = 'input_files'
+FILENAME = 'filename'
 FX_FILES = 'fx_files'
 METADATA_YAML_FILE = 'metadata.yml'
 OUTPUT_FILE_TYPE = 'output_file_type'
@@ -40,5 +41,3 @@ PLOT_DIR = 'plot_dir'
 SCRIPT = 'script'
 VERSION = 'version'
 WORK_DIR = 'work_dir'
-WRITE_NETCDF = 'write_netcdf'
-WRITE_PLOTS = 'write_plots'
